window.Gleam.initSearch([{"doc":"glitr_convert","title":"glitr_convert","content":"# glitr_convert\n\n[![Package Version](https://img.shields.io/hexpm/v/glitr_convert)](https://hex.pm/packages/glitr_convert)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glitr_convert/)\n\n**Encode and decode from and to Gleam types effortlessly !**\n\nDefine a converter once and encode and decode as much as you want.\n\n## Installation\n\n```sh\ngleam add glitr_convert\n```\n\n## Usage\n\n```gleam\nimport glitr_convert/converter as c\nimport gleam/json\n\ntype Person {\n  Person(name: String, age: Int)\n}\n\npub fn main() {\n  let converter = c.object({\n    use name &lt;- c.parameter\n    use age &lt;- c.parameter\n    use &lt;- c.constructor\n    Person(name:, age:)\n  })\n  |&gt; field(&quot;name&quot;, fn(v) { Ok(v.name) }, c.string())\n  |&gt; field(&quot;age&quot;, fn(v) { Ok(v.age) }, c.int())\n  |&gt; c.to_converter\n\n  converter\n  |&gt; c.json_encode(Person(&quot;Anna&quot;, 21))\n  |&gt; json.to_string\n  // &#39;{&quot;name&quot;: &quot;Anna&quot;, &quot;age&quot;: 21}&#39;\n\n  &quot;{\\&quot;name\\&quot;: \\&quot;Bob\\&quot;, \\&quot;age\\&quot;: 36}&quot;\n  |&gt; json.decode(c.json_decode(converter, _))\n  // Ok(Person(&quot;Bob&quot;, 36))\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/glitr_convert&gt;.\n\n## Features\n\n- Javascript and Erlang targets\n- Converters for all basic types (except BitArray)\n- Define converters for List, Dict, Result &amp; Option\n- Build decoders for custom objects and enums\n- Encode and decode to JSON.\n\n## Potential developments\n\n- Add BitArray support\n- Add Yaml conversion\n\nFeel free to open PRs and issues if you want more features !\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"glitr_convert","title":"GlitrType","content":"pub type GlitrType {\n  String\n  Bool\n  Float\n  Int\n  Null\n  List(of: GlitrType)\n  Dict(key: GlitrType, value: GlitrType)\n  Object(fields: List(#(String, GlitrType)))\n  Optional(of: GlitrType)\n  Result(result: GlitrType, error: GlitrType)\n  Enum(variants: List(#(String, GlitrType)))\n}\n This type is used to define the shape of the data.  \n It isn&#39;t meant to be used directly !  \n It is better to use converters that use GlitrTypes internally to decode data.\nString\n\n\nBool\n\n\nFloat\n\n\nInt\n\n\nNull\n\n\nList(of: GlitrType)\n\n\nDict(key: GlitrType, value: GlitrType)\n\n\nObject(fields: List(#(String, GlitrType)))\n\n\nOptional(of: GlitrType)\n\n\nResult(result: GlitrType, error: GlitrType)\n\n\nEnum(variants: List(#(String, GlitrType)))\n\n\nSynonyms:\nglitr_convert.GlitrType\nglitr_convert GlitrType","url":"glitr_convert.html#GlitrType"},{"doc":"glitr_convert","title":"GlitrValue","content":"pub type GlitrValue {\n  StringValue(value: String)\n  BoolValue(value: Bool)\n  FloatValue(value: Float)\n  IntValue(value: Int)\n  NullValue\n  ListValue(value: List(GlitrValue))\n  DictValue(value: dict.Dict(GlitrValue, GlitrValue))\n  ObjectValue(value: List(#(String, GlitrValue)))\n  OptionalValue(value: option.Option(GlitrValue))\n  ResultValue(value: Result(GlitrValue, GlitrValue))\n  EnumValue(variant: String, value: GlitrValue)\n}\n This type is used to represent data values.  \n It is an intermediate type between encoded data and Gleam types.\n It isn&#39;t meant to be used directly !  \nStringValue(value: String)\n\n\nBoolValue(value: Bool)\n\n\nFloatValue(value: Float)\n\n\nIntValue(value: Int)\n\n\nNullValue\n\n\nListValue(value: List(GlitrValue))\n\n\nDictValue(value: dict.Dict(GlitrValue, GlitrValue))\n\n\nObjectValue(value: List(#(String, GlitrValue)))\n\n\nOptionalValue(value: option.Option(GlitrValue))\n\n\nResultValue(value: Result(GlitrValue, GlitrValue))\n\n\nEnumValue(variant: String, value: GlitrValue)\n\n\nSynonyms:\nglitr_convert.GlitrValue\nglitr_convert GlitrValue","url":"glitr_convert.html#GlitrValue"},{"doc":"glitr_convert","title":"json_decode","content":"pub fn json_decode(\n  of: GlitrType,\n) -&gt; fn(Dynamic) -&gt; Result(GlitrValue, List(DecodeError))\n Decode a JSON value using the specified GlitrType as the shape of the data.  \n Returns the corresponding GlitrValue representation.\n This isn&#39;t meant to be used directly !\nSynonyms:\nglitr_convert.json_decode\nglitr_convert json_decode","url":"glitr_convert.html#json_decode"},{"doc":"glitr_convert","title":"json_encode","content":"pub fn json_encode(val: GlitrValue) -&gt; Json\n Encode a GlitrValue into its corresponding JSON representation.  \n This is not meant to be used directly !  \n It is better to use converters.\nSynonyms:\nglitr_convert.json_encode\nglitr_convert json_encode","url":"glitr_convert.html#json_encode"},{"doc":"glitr_convert","title":"glitr_convert","content":"","url":"glitr_convert.html"},{"doc":"glitr_convert/converter","title":"Converter","content":"pub opaque type Converter(a)\n A converter is an object with the data necessary to encode and decode a specific Gleam type.  \n You can build converters using the provided constructors.\n\nSynonyms:\nglitr_convert/converter.Converter\nglitr_convert/converter Converter","url":"glitr_convert/converter.html#Converter"},{"doc":"glitr_convert/converter","title":"ObjectConverter","content":"pub opaque type ObjectConverter(current, base)\n This is an intermediary type to build converters for a custom Gleam type.  \n TODO: rename\n\nSynonyms:\nglitr_convert/converter.ObjectConverter\nglitr_convert/converter ObjectConverter","url":"glitr_convert/converter.html#ObjectConverter"},{"doc":"glitr_convert/converter","title":"ObjectWithFieldConverter","content":"pub opaque type ObjectWithFieldConverter(current, base)\n This is an intermediary type to build converters for a custom Gleam type.  \n TODO: rename\n\nSynonyms:\nglitr_convert/converter.ObjectWithFieldConverter\nglitr_convert/converter ObjectWithFieldConverter","url":"glitr_convert/converter.html#ObjectWithFieldConverter"},{"doc":"glitr_convert/converter","title":"bool","content":"pub fn bool() -&gt; Converter(Bool)\n Basic converter for a Bool value\nSynonyms:\nglitr_convert/converter.bool\nglitr_convert/converter bool","url":"glitr_convert/converter.html#bool"},{"doc":"glitr_convert/converter","title":"constructor","content":"pub fn constructor(c: fn() -&gt; a) -&gt; ObjectConverter(Nil, a)\n Specify that the next instruction returns a constructed instance of the type to convert.  \n See `object()`\nSynonyms:\nglitr_convert/converter.constructor\nglitr_convert/converter constructor","url":"glitr_convert/converter.html#constructor"},{"doc":"glitr_convert/converter","title":"decode","content":"pub fn decode(\n  converter: Converter(a),\n  value: GlitrValue,\n) -&gt; Result(a, List(DecodeError))\n Decode a GlitrValue using the provided converter.\nSynonyms:\nglitr_convert/converter.decode\nglitr_convert/converter decode","url":"glitr_convert/converter.html#decode"},{"doc":"glitr_convert/converter","title":"dict","content":"pub fn dict(\n  key: Converter(a),\n  value: Converter(b),\n) -&gt; Converter(Dict(a, b))\n Basic converter for a Dict value.\n \n `key` is a converter for the keys.\n `value` is a converter for the values.\n \n Example:\n \n let converter: Converter(Dict(String, Int)) = dict(string(), int())\n \nSynonyms:\nglitr_convert/converter.dict\nglitr_convert/converter dict","url":"glitr_convert/converter.html#dict"},{"doc":"glitr_convert/converter","title":"encode","content":"pub fn encode(converter: Converter(a), value: a) -&gt; GlitrValue\n Encode a value into the corresponding GlitrValue using the converter.  \n If the converter isn&#39;t valid, a NullValue is returned.\nSynonyms:\nglitr_convert/converter.encode\nglitr_convert/converter encode","url":"glitr_convert/converter.html#encode"},{"doc":"glitr_convert/converter","title":"enum","content":"pub fn enum(\n  tags: fn(a) -&gt; String,\n  converters: List(#(String, Converter(a))),\n) -&gt; Converter(a)\n Create a converter for an enum type\n \n `tags` is a function that associate a tag to each variant of the enum\n `converters` is a list of converters, each associated with a tag\n \n Example:\n \n type Action {\n   Open(id: String)\n   Close(id: String)\n }\n \n let open_converter = object({\n   use id &lt;- parameter\n   use &lt;- constructor\n   Open(id:)\n })\n |&gt; field(&quot;id&quot;, fn(v) {\n   case v {\n     Open(id) -&gt; Ok(id)\n     _ -&gt; Error(Nil)\n   }\n })\n |&gt; to_converter\n \n let close_converter = object({\n   use id &lt;- parameter\n   use &lt;- constructor\n   Close(id:)\n })\n |&gt; field(&quot;id&quot;, fn(v) {\n   case v {\n     Close(id) -&gt; Ok(id)\n     _ -&gt; Error(Nil)\n   }\n })\n |&gt; to_converter\n \n let action_converter = enum(\n   fn(v) {\n     case v {\n       Open(_) -&gt; &quot;Open&quot;\n       Close(_) -&gt; &quot;Close&quot;\n     }\n   },\n   [\n     #(&quot;Open&quot;, open_converter),\n     #(&quot;Close&quot;, close_converter),\n   ]\n )\n \nSynonyms:\nglitr_convert/converter.enum\nglitr_convert/converter enum","url":"glitr_convert/converter.html#enum"},{"doc":"glitr_convert/converter","title":"field","content":"pub fn field(\n  converter: ObjectWithFieldConverter(#(a, b), c),\n  field_name: String,\n  field_getter: fn(c) -&gt; Result(a, Nil),\n  field_type: Converter(a),\n) -&gt; ObjectWithFieldConverter(b, c)\n Provide information about the fields of an object converter. \n  \n `field_name` is the key that will be used in the encoded data.  \n `field_getter` is a function returning a way to access the field from an instance.  \n `field_type` is the converter to use for this field.  \n \n See `object()` for an example.\nSynonyms:\nglitr_convert/converter.field\nglitr_convert/converter field","url":"glitr_convert/converter.html#field"},{"doc":"glitr_convert/converter","title":"float","content":"pub fn float() -&gt; Converter(Float)\n Basic converter for a Float value\nSynonyms:\nglitr_convert/converter.float\nglitr_convert/converter float","url":"glitr_convert/converter.html#float"},{"doc":"glitr_convert/converter","title":"int","content":"pub fn int() -&gt; Converter(Int)\n Basic converter for a Int value\nSynonyms:\nglitr_convert/converter.int\nglitr_convert/converter int","url":"glitr_convert/converter.html#int"},{"doc":"glitr_convert/converter","title":"json_decode","content":"pub fn json_decode(\n  converter: Converter(a),\n  value: Dynamic,\n) -&gt; Result(a, List(DecodeError))\n Decode a Json value using the provided converter.\nSynonyms:\nglitr_convert/converter.json_decode\nglitr_convert/converter json_decode","url":"glitr_convert/converter.html#json_decode"},{"doc":"glitr_convert/converter","title":"json_encode","content":"pub fn json_encode(converter: Converter(a), value: a) -&gt; Json\n Encode a value into the corresponding Json using the converter.  \n If the converter isn&#39;t valid, a NullValue is returned.\nSynonyms:\nglitr_convert/converter.json_encode\nglitr_convert/converter json_encode","url":"glitr_convert/converter.html#json_encode"},{"doc":"glitr_convert/converter","title":"list","content":"pub fn list(of: Converter(a)) -&gt; Converter(List(a))\n Basic converter for a List value.   \n \n `of` is a converter for the type of the elements.\nSynonyms:\nglitr_convert/converter.list\nglitr_convert/converter list","url":"glitr_convert/converter.html#list"},{"doc":"glitr_convert/converter","title":"null","content":"pub fn null() -&gt; Converter(Nil)\n Basic converter for a Nil value\nSynonyms:\nglitr_convert/converter.null\nglitr_convert/converter null","url":"glitr_convert/converter.html#null"},{"doc":"glitr_convert/converter","title":"object","content":"pub fn object(\n  object_converter: ObjectConverter(a, b),\n) -&gt; ObjectWithFieldConverter(a, b)\n Build a converter for a custom Gleam type.\n \n Example: \n \n type Person {\n   Person(name: String, age: Int)\n }\n \n let converter = object({\n   use name &lt;- parameter\n   use age &lt;- parameter\n   use &lt;- constructor\n \n   Person(name:, age:)\n })\n |&gt; field(&quot;name&quot;, fn(v) { Ok(v.name) }, string())\n |&gt; field(&quot;age&quot;, fn(v) { Ok(v.age) }, int())\n |&gt; to_converter\n \nSynonyms:\nglitr_convert/converter.object\nglitr_convert/converter object","url":"glitr_convert/converter.html#object"},{"doc":"glitr_convert/converter","title":"optional","content":"pub fn optional(of: Converter(a)) -&gt; Converter(Option(a))\n Basic converter for a Option value.\n \n `of` is a converter for the optional value.\nSynonyms:\nglitr_convert/converter.optional\nglitr_convert/converter optional","url":"glitr_convert/converter.html#optional"},{"doc":"glitr_convert/converter","title":"parameter","content":"pub fn parameter(\n  next: fn(a) -&gt; ObjectConverter(b, c),\n) -&gt; ObjectConverter(#(a, b), c)\n Specify a new parameter to be used in an object converter.  \n See `object()`\nSynonyms:\nglitr_convert/converter.parameter\nglitr_convert/converter parameter","url":"glitr_convert/converter.html#parameter"},{"doc":"glitr_convert/converter","title":"result","content":"pub fn result(\n  res: Converter(a),\n  error: Converter(b),\n) -&gt; Converter(Result(a, b))\n Basic converter for a Result value.\n \n `res` is a converter for the Ok value.\n `error` is a converter for the Error value.\nSynonyms:\nglitr_convert/converter.result\nglitr_convert/converter result","url":"glitr_convert/converter.html#result"},{"doc":"glitr_convert/converter","title":"string","content":"pub fn string() -&gt; Converter(String)\n Basic converter for a String value\nSynonyms:\nglitr_convert/converter.string\nglitr_convert/converter string","url":"glitr_convert/converter.html#string"},{"doc":"glitr_convert/converter","title":"to_converter","content":"pub fn to_converter(\n  converter: ObjectWithFieldConverter(Nil, a),\n) -&gt; Converter(a)\n Generate a converter from a builder type\nSynonyms:\nglitr_convert/converter.to_converter\nglitr_convert/converter to_converter","url":"glitr_convert/converter.html#to_converter"},{"doc":"glitr_convert/converter","title":"glitr_convert/converter","content":"","url":"glitr_convert/converter.html"}]);